@@ -2,7 +2,7 @@
 
 /*
  *  pictor: a web application for sharing, viewing, and organizing pictures
- *  Copyright (C) 1997-2013 Dustin Kirkland <dustin.kirkland@gmail.com>
+ *  Copyright (C) 1997-2017 Dustin Kirkland <dustin.kirkland@gmail.com>
  *
  *  This program is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Affero General Public License as published by
@@ -29,10 +29,10 @@ $width       = sanity_check_number("width");
 $rotate      = sanity_check_number("rotate");
 $base        = sanity_check("base");
 $thumbs      = sanity_check_number("thumbs");
+$page        = sanity_check_number("page");
 $slideshow   = sanity_check_number("slideshow");
 $write       = sanity_check("write");
 $file        = sanity_check_array("file");
-$desc        = sanity_check_array("desc");
 $random      = sanity_check_number("random");
 $screensaver = sanity_check_number("screensaver");
 
@@ -40,6 +40,13 @@ if (!isset($DEFAULT_WIDTH)) {
 	$DEFAULT_WIDTH = 800;
 }
 
+if (!isset($thumbs) || $thumbs <= 1) {
+	$thumbs = 50;
+}
+if (!isset($page)) {
+	$page = 0;
+}
+
 /* check for malicious .. in $base input */
 $PICTURE_ROOT = "pictures";
 $BASEDIR = $PICTURE_ROOT . "/" . $base;
@@ -48,6 +55,7 @@ $THUMB_ROOT = $PICTURE_ROOT;
 $BATCH = false;
 if (isset($argv) && $argv[1] == "batch") {
 	$BATCH = true;
+	set_time_limit(0);
 }
 
 /****************************************************************************/
@@ -125,28 +133,6 @@ function has_images($dir) {
 /****************************************************************************/
 
 
-/****************************************************************************/
-/* Read and parse the description */
-function get_description($path) {
-	$path .= "/";
-	$dirs = preg_split("/\//", $path);
-	$current = array_shift($dirs);
-	$description = array();
-	while (sizeof($dirs) > 0) {
-		if ( file_exists("$current/description.txt") ) {
-			$file = file("$current/description.txt");
-			for ($i=0; $i<sizeof($file); $i++) {
-				$line = preg_split("/\t+/", $file[$i], 2);
-				$description[$line[0]] = rtrim($line[1]);
-			}
-		}
-		$current = $current . "/" . array_shift($dirs);
-	}
-	return $description;
-}
-/****************************************************************************/
-
-
 /****************************************************************************/
 /* Given a filename, return true if has an image file extension */
 function is_image_filename($file) {
@@ -271,6 +257,8 @@ function do_resize_picture($path_to_picture, $width, $height, $rotate) {
 /****************************************************************************/
 /* Transcode a video, returns temp file name, depends on libav-tools */
 function do_transcode_video($path) {
+	return $path;
+/*
 	$path_parts = preg_split("/\//", $path);
 	$file = array_pop($path_parts);
         $tempfilename = get_cache_filename($file, "transcode");
@@ -289,6 +277,7 @@ function do_transcode_video($path) {
 		//clean_tmp("tmp/transcode");
 		return $tempfilename;
 	}
+*/
 }
 /****************************************************************************/
 
@@ -324,7 +313,7 @@ function goto(form) {
 <style>
 td {
 	text-decoration: none;
-	font-size: 8px
+	font-size: 10px
 	border: 1px black;
 	-moz-border-radius: 10px;
 	border-radius: 10px;
@@ -336,10 +325,12 @@ a {
 a:hover {
 	text-decoration: underline;
 }
+p {
+	font-size: 10px
+}
 body {
-	font-size: 8px;
-	font-family: verdana,arial,helvetica,sans-serif;
-	font-weight: 400;
+	font-family: Ubuntu,verdana,arial,helvetica,sans-serif;
+	font-size: 10px;
 	color: black;
 }
 div {
@@ -351,7 +342,8 @@ div {
 	top:0;
 	right:0;
 }
-</style>
+img {
+border-radius:10px
 </style>
 <title>$TITLE</title>
 <link rel='shortcut icon' href='/pictor/favicon.ico' type='image/x-icon'>
@@ -360,7 +352,6 @@ div {
 	if ($body == 1) {
 		print("
 <body topmargin=0 leftmargin=0 rightmargin=0 bottommargin=0 bgcolor=#101010>
-<table width=100% height=100%><tr><td align=center valign=center>
 		");
 	}
 }
@@ -370,20 +361,18 @@ div {
 /****************************************************************************/
 /* Print html footer information */
 function print_footer() {
-	global $LICENSE;
 	print("
-		<table border=0 cellspacing=0 width=400 align=center>
-		  <tr>
-		    <td bgcolor=#FFFFFF align=center><small>
-$LICENSE<br>
-<small><a href='https://launchpad.net/pictor'>Pictor</a> is <a href='agpl-3.0.txt'>AGPLv3</a> free software, Copyright &copy; 1997-2013 <a href='http://blog.dustinkirkland.com'>Dustin Kirkland</a>.</small>
-		    </small></td>
-		  </tr>
-		</table>
-</td></tr></table>
+<br/>
+<table border=0 cellpadding=4 width=100% align=center>
+  <tr>
+    <td bgcolor=#DDDDDD align=center>
+      <p><a href='https://launchpad.net/pictor'>Pictor</a> is <a href='agpl-3.0.txt'>AGPLv3</a> free software, Copyright &copy; 1997-" . date("Y") . " <a href='http://blog.dustinkirkland.com'>Dustin Kirkland</a>.</p>
+    </td>
+  </tr>
+</table>
 </body>
 </html>
-	");
+");
 }
 /****************************************************************************/
 
@@ -457,9 +446,6 @@ function generate_thumbnail($filename) {
 		}
 		rotate_if_necessary($filename, $thumbnail_name);
 	} elseif (is_video($filename)) {
-		if ($BATCH) {
-			echo "Rendering thumbnail [$filename] to [$thumbnail_name]...\n";
-		}
 		if (! shell_exec("HOME=/var/cache/pictor/ run-one avconv -i " . escapeshellarg("$filename") . " -r 1 -ss 00:00:10 -f image2 -vframes 1 -vf 'scale=130:trunc(ow/a/2)*2' -y " . escapeshellarg("$thumbnail_name"))) {
 			symlink(getcwd() . "/pictor.jpg", $thumbnail_name);
 		}
@@ -470,13 +456,13 @@ function generate_thumbnail($filename) {
 
 /****************************************************************************/
 /* Print single thumbnail */
-function print_thumbnail($path, $file, $desc) {
+function print_thumbnail($path, $file) {
 	global $THUMB_ROOT;
 	global $PICTURE_ROOT;
 	$filename = "$PICTURE_ROOT/$path/$file";
 	$thumbnail_name = get_cache_filename($filename, "thumbnails");
 	$href = "?album=" . urlencode($path) . "&picture=" . urlencode($file);
-	print("<a title='$desc' href='$href'>");
+	print("<a href='$href'>");
 	generate_thumbnail($filename);
 	print("<div><img height=130 align=center border=0 src='$thumbnail_name'>");
 	if (is_video($filename)) {
@@ -489,62 +475,52 @@ function print_thumbnail($path, $file, $desc) {
 
 /****************************************************************************/
 /* List albums */
-function do_list_albums($base) {
+function do_list_albums($album, $thumbs, $page) {
 	global $BASEDIR;
 	global $ALBUM_COLUMNS;
-	if ($dir = @opendir($BASEDIR . "/" . $base)) {
+	$pictures = get_pictures_from_album($album);
+	if ($dir = @opendir($BASEDIR . "/" . $album)) {
 		$i = 0;
 		while (($file = readdir($dir)) !== false) {
 			$files[$i++] = $file;
 		}
 		closedir($dir);
 		sort($files);
-		if ($base) {
-			$header = preg_replace("/^\//", "", $base);
-			$header = preg_replace("/\//", " - ", $header);
-		} else {
-			$header = "All Albums";
-			print_upper_banner("", "", 600);
-		}
+		print_banner($album, $thumbs, $page, "");
 		print("
-<table border=0 cellspacing=0 cellpadding=10 align=center>
-  <tr>
-    <td bgcolor=#EEEEEE>
-      <table border=0 cellspacing=4 cellpadding=2 align=center>
-        <tr>
-          <td colspan=$ALBUM_COLUMNS bgcolor=#DDDDDD align='center'><b>$header</b></td>
-        </tr>
-		");
+<table border=0 cellpadding=4 cellspacing=4 align=center>");
 		$count = 0;
 		for ($i=0; $i<sizeof($files); $i++) {
 			$file = $files[$i];
-			if ( is_dir("$BASEDIR/$base/$file") && (!preg_match("/^[_\.]/", $file))) {
-				$href = "?album=" . urlencode("$base/$file") . "&thumbs=1";
+			if (is_dir("$BASEDIR/$album/$file") && (!preg_match("/^[_\.]/", $file))) {
+				$href = "?album=" . urlencode("$album/$file") . "&thumbs=" . urlencode($thumbs);
 				if ( ($count % $ALBUM_COLUMNS) == 0 ) {
 					print("
-        <tr>
-					");
+  <tr>");
 				}
 				print("
-          <td bgcolor=white onMouseOver=this.bgColor='lightblue' onMouseOut=this.bgColor='white' onClick=javascript:goto('$href') align=center><a href='$href'>" . htmlspecialchars($file) . "</a></td>
-				");
+    <td bgcolor=#FFFFFF onMouseOver=this.bgColor='lightblue' onMouseOut=this.bgColor='white' onClick=javascript:goto('$href') align=center><a href='$href'>" . htmlspecialchars($file) . "</a></td>");
 				if ( (($count+1) % $ALBUM_COLUMNS) == 0 ) {
 					print("
-        </tr>
-					");
+  </tr>");
 				}
 				$count++;
 			}
 		}
 		print("
-      </table>
+</table>
+");
+	} else {
+		print("
+<table border=0 cellpadding=20>
+  <tr>
+    <td bgcolor=#EEEEEE>
+      <br><b>ERROR</b><br>No pictures found.<br><br>Create a symlink to your pictures folder at<pre>" . dirname($_SERVER["SCRIPT_FILENAME"]) . "/pictures</pre>
     </td>
   </tr>
 </table>
-		");
-	} else {
-		print("<table cellpadding=20><tr><td bgcolor=#EEEEEE><br><b>ERROR</b><br>No pictures found.<br><br>Create a symlink to your pictures folder at<pre>" . dirname($_SERVER["SCRIPT_FILENAME"]) . "/pictures</pre></td></tr></table>");
-                exit;
+");
+		//exit;
 	}
 	// if only one option and no images, go straight to it
 	if ($count == 1) {
@@ -561,49 +537,57 @@ function do_list_albums($base) {
 
 /****************************************************************************/
 /* Print album thumbails */
-function print_thumbnails($album) {
+function print_thumbnails($album, $thumbs, $page) {
 	global $BASEDIR;
 	print("<br>\n");
 	$pictures = get_pictures_from_album($album);
-	$description = get_description("$BASEDIR/$album");
 	$tab = 1;
-	do_list_albums($album);
-	print("<table align=center><tr><td bgcolor=#888888><center>\n");
-	for ($i=0; $i<sizeof($pictures); $i++) {
-		if (isset($description[$pictures[$i]])) {
-			$desc = $description[$pictures[$i]];
-		} else {
-			$desc = "";
+	do_list_albums($album, $thumbs, $page);
+	print("
+<table border=0 align=center>
+  <tr>
+    <td bgcolor=#888888><center>
+\n");
+	if ($page > 0) {
+		$url = "?album=" . urlencode($album) . "&thumbs=" . urlencode($thumbs) . "&page=" . urlencode($page-1);
+		print("
+      <table border=0 align=left>
+        <tr>
+          <td width=300 height=50 bgcolor=#DDDDDD align=center onClick=javascript:gotourl('$url')>
+            <a href='$url'> Previous " . (($page-1)*$thumbs+1) . "-" . $page*$thumbs . " of " . sizeof($pictures) . " images</a></center>
+          </td>
+        </tr>
+      </table>
+");
+	}
+	$start = $page * $thumbs;
+	$stop = $start + $thumbs;
+	for ($i=$start; $i<$stop; $i++) {
+		if (! isset($pictures[$i])) {
+			break;
 		}
-		print_thumbnail($album, $pictures[$i], $desc);
+		print_thumbnail($album, $pictures[$i]);
 	}
-	print("</center></td></tr></table>");
-}
-/****************************************************************************/
-
-
-/****************************************************************************/
-/* Write description file */
-/* Descriptions are typically written to description.txt
-   Descriptions written via the web interface are written to
-   description.txt which must be chowned to www-data:www-data
-*/
-function do_write_descriptions($album, $file, $desc) {
-	print("<table bgcolor=white border=0><tr><td align=left><pre>\n");
-	print("FILENAME: $album/description.txt<br><br>\n");
-	if (!$fh = fopen("pictures/$BASEDIR/$album/description.txt", "w")) {
-		print("Cannot open [$BASEDIR/$album/description.txt]\n");
-		exit;
+	if ($i < sizeof($pictures)) {
+		$url = "?album=" . urlencode($album) . "&thumbs=" . urlencode($thumbs) . "&page=" . urlencode($page+1);
+		print("
+<br>
+      <table border=0 align=right>
+        <tr>
+          <td width=300 height=50 bgcolor=#DDDDDD align=center onClick=javascript:gotourl('$url')>
+            <a href='$url'> Next ". (($page+1)*$thumbs+1) . "-" . min(($page+2)*$thumbs, sizeof($pictures)) . " of " . sizeof($pictures) . " images</a>
+          </td>
+        </tr>
+      </table>
+");
 	}
-	for ($i=0; $i<sizeof($file); $i++) {
-		if ($desc[$i]) {
-			print("$file[$i]\t\t$desc[$i]\n");
-			fwrite($fh, "$file[$i]\t\t$desc[$i]\n");
-		}
-	}
-	fclose($fh);
-	print("</pre></td></tr></table>");
-	print("<br><a href='?'>index</a> | <a href='?thumbs=1&album=". urlencode($album) . "'>thumbs</a> | <a href='?album=" . urlencode($album) . "'>flipbook</a><br><br>");
+	print("
+</center>
+    </td>
+  </tr>
+</table>
+");
+	print_banner($album, $thumbs, $page, "");
 }
 /****************************************************************************/
 
@@ -668,21 +652,17 @@ function build_delay_form($path_to_picture, $album, $picture, $width, $delay) {
 
 /****************************************************************************/
 /* Print picture form */
-function build_picture_form($album, $picture, $width, $slideshow, $pictures, $description) {
+function build_picture_form($album, $picture, $width, $slideshow, $pictures) {
 	global $BASEDIR;
 	$form = "<form name=picform><select name=dest size=1 onchange=javascript:goto(this.form)>\n";
 	if (is_dir("$BASEDIR/$album")) {
 		$i = 0;
 		for ($i=0; $i<sizeof($pictures); $i++) {
 			$file = $pictures[$i];
-			if (isset($description[$file])) {
-				$clean = substr($description[$file], 0, round($width/13));
-			} else {
-				$clean = preg_replace("/\.jpg$/i", "", $file);
-				$clean = preg_replace("/[\-\_]/", " ", $clean);
-				$clean = preg_replace("/&/", " and ", $clean);
-				$clean = htmlspecialchars($clean);
-			}
+			$clean = preg_replace("/\.jpg$/i", "", $file);
+			$clean = preg_replace("/[\-\_]/", " ", $clean);
+			$clean = preg_replace("/&/", " and ", $clean);
+			$clean = htmlspecialchars($clean);
 			if ($file == $picture) {
 				$currentindex = $i;
 				$form .= "<option selected value>$clean</option>\n";
@@ -710,7 +690,14 @@ function get_next_link($album, $width, $pictures, $currentindex, $slideshow) {
 	if ($slideshow > 0) {
 		print("<meta http-equiv='refresh' content='$slideshow;url=$NEXT&slideshow=$slideshow'>\n");
 	}
-	$next = "<table border=0><tr><td bgcolor=white><a href='$NEXT'><b>Next <img src=silk/resultset_next.png></b></a></td></tr></table>";
+	$next = "
+<table border=0>
+  <tr>
+    <td bgcolor=white>
+      <a href='$NEXT'><b>Next <img src=silk/resultset_next.png></b></a>
+    </td>
+  </tr>
+</table>";
 	return $next;
 }
 
@@ -719,7 +706,15 @@ function get_back_link($album, $width, $pictures, $currentindex) {
 	if ($currentindex != 0) {
 		$back = $pictures[$currentindex-1];
 		$BACK = "?album=" . urlencode($album) . "&picture=" . urlencode($back) . "&width=$width";
-		$back = "<table border=0><tr><td bgcolor=white><a href='$BACK'><b><img src=silk/resultset_previous.png> Back</b></a></td></tr></table>";
+		$back = "
+<table border=0>
+  <tr>
+    <td bgcolor=white>
+      <a href='$BACK'><b><img src=silk/resultset_previous.png> Back</b></a>
+    </td>
+  </tr>
+</table>
+";
 		return $back;
 	} else {
 		return "&nbsp;";
@@ -732,7 +727,13 @@ function get_back_link($album, $width, $pictures, $currentindex) {
 /* Print data cell */
 function print_data_cell($key, $value) {
 	if ($value) {
-		print("<tr><td bgcolor=#DDDDDD><small><small>" . htmlspecialchars($key) . "</small></small></td><td bgcolor=white><small><small>" . htmlspecialchars($value) . "</small></small></td></tr>");
+		print("<tr><td bgcolor=#DDDDDD>" . htmlspecialchars($key) . "</td><td bgcolor=white>");
+		if (is_array($value)) {
+			print(htmlspecialchars(print_r($value)));
+		} else {
+			print(htmlspecialchars($value));
+		}
+		print("</td></tr>");
 	}
 }
 /****************************************************************************/
@@ -740,99 +741,86 @@ function print_data_cell($key, $value) {
 
 /****************************************************************************/
 /* Print exif data */
-function print_exif_data($path_to_picture, $description) {
-	$keys = array();
-	$values = array();
-	array_push($keys, "File name"); array_push($values, basename($path_to_picture));
-	array_push($keys, "File size"); array_push($values, round(filesize($path_to_picture)/1024)." KB");
+function print_exif_data($path_to_picture) {
 	if ($exif = @exif_read_data($path_to_picture, 0, false)) {
-		array_push($keys, "Resolution"); array_push($values, $exif["ExifImageWidth"] . "x" . $exif["ExifImageLength"]);
-		array_push($keys, "Date/Time"); array_push($values, $exif["DateTimeOriginal"]);
-		array_push($keys, "Camera model"); array_push($values, $exif["Model"]);
-		//foreach ($exif as $key => $section) {
-		//    echo "$key -- $section <br>";
-		//}
-	}
-
-	print("<center>");
-	print("<table border=0 cellspacing=5><tr valign=top><td>");
-	print("<table border=0 cellspacing=2>");
-	for ($i=0; $i<sizeof($keys); $i++) {
-		print_data_cell($keys[$i], $values[$i]);
-	}
-	print("</table></td><td>");
-	$keys = array("Flash", "FocalLength", "ExposureTime", "ApertureValue", "ISOSpeedRatings", "WhiteBalance", "MeteringMode");
-	print("<table border=0 cellspacing=1>");
-	for ($i=0; $i<sizeof($keys); $i++) {
-		if (isset($keys[$i]) && isset($exif[$keys[$i]])) {
-			print_data_cell($keys[$i], $exif[$keys[$i]]);
+		print("<center>");
+		print("
+<table border=0 cellspacing=2 cellpadding=2>
+");
+		foreach ($exif as $key => $section) {
+			print_data_cell($key, $section);
 		}
+		print("
+</table>
+");
 	}
-	print("</table></td></tr></table></center>");
 }
 /****************************************************************************/
 
 
 /****************************************************************************/
 /* Print picture details */
-function print_picture_details($path_to_picture, $currentindex, $total, $width, $description) {
-	print_exif_data($path_to_picture, $description);
+function print_picture_details($path_to_picture, $currentindex, $total, $width) {
+	print_exif_data($path_to_picture);
 }
 /****************************************************************************/
 
 
 /****************************************************************************/
-/* Print upper banner */
-function print_upper_banner($album, $description, $width) {
-	global $TITLE;
-	$descr = preg_replace("/^\//", "", $album);
-	$path_parts = preg_split("/\/+/", $descr);
-	$descr = "";
+/* Print banner */
+function print_banner($album, $thumbs, $page, $picture) {
+	global $TITLE, $BASEDIR;
+	$pictures = array();
+	$pictures = get_pictures_from_album($album);
+	$links = preg_replace("/^\//", "", $album);
+	$path_parts = preg_split("/\/+/", $links);
+	$links = "<a href='?album='>All Albums</a> - ";
 	$subalbum = "";
 	for ($i=0; $i<sizeof($path_parts)-1; $i++) {
 		$subalbum .= "/" . $path_parts[$i];
-		$descr .= "<a href='?thumbs=1&album=" . urlencode($subalbum) . "'>$path_parts[$i]</a> - ";
+		$links .= "<a href='?thumbs=". urlencode($thumbs) . "&album=" . urlencode($subalbum) . "'>$path_parts[$i]</a> - ";
 	}
 	$subalbum .= "/" . $path_parts[$i];
-	$descr .= "<a href='?album=" . urlencode($subalbum) . "'>$path_parts[$i]</a>";
-	if ($description) {
-		$descr .= " - $description";
+	$links .= "<a href='?album=" . urlencode($subalbum) . "'>$path_parts[$i]</a>";
+	$prev = "&nbsp;";
+	$next = "&nbsp;";
+	if ($album && $page > 0) {
+		$prev = "<a href=?album=" . urlencode($album) . "&thumbs=" . urlencode($thumbs) . "&page=" . urlencode($page-1) . "><img src=silk/resultset_previous.png><b>Back</b></a>";
+	}
+	if ($album && sizeof($pictures) > $thumbs * ($page + 1)) {
+		$next = "<a href=?album=" . urlencode($album) . "&thumbs=" . urlencode($thumbs) . "&page=" . urlencode($page+1) . "><b>Next</b> <img src=silk/resultset_next.png></a>";
+	}
+	if ($picture) {
+		$path_to_picture = $BASEDIR . "/" . $album . "/" . $picture;
+		$links .= " - <a href='$path_to_picture'>$picture</a>";
+		$index = array_search($picture, $pictures);
+		if (isset($pictures[$index-1])) {
+			$prev = "<a href=?album=" . urlencode($album) . "&picture=" . $pictures[$index-1] . "><img src=silk/resultset_previous.png><b>Back</b></a>";
+		}
+		if (isset($pictures[$index+1])) {
+			$next = "<a href=?album=" . urlencode($album) . "&picture=" . $pictures[$index+1] . "><b>Next</b> <img src=silk/resultset_next.png></a>";
+		}
 	}
-	print("<table border=0 cellpadding=0 cellspacing=0 align=center width=$width><tr><td bgcolor=white>
-		  <table border=0 cellpadding=0 cellspacing=0 align=center width=100%><tr><td>
-                   <table border=0 cellpadding=0 cellspacing=0 align=center width=100%>
-		    <tr>
-		     <td colspan=3>
-		      <p align=center><b>$descr</b></p>
-		     </td>
-		    </tr>
-	");
-}
-/****************************************************************************/
-
-
-/****************************************************************************/
-/* Print upper toolbar */
-function print_upper_toolbar($album, $description, $back, $next, $width) {
-	print_upper_banner($album, $description, $width);
 	print("
-		  <tr align=center>
-		    <td width=20% align=left>$back</td>
-		    <td width=60%>
-		      <a href='" . $_SERVER["PHP_SELF"] . "'>index</a> |
-		      <a href='?album=" . urlencode($album) . "&thumbs=1'>thumbs</a> |
-		      <a href='?album=" . urlencode($album) . "&width=$width&slideshow=4'>slideshow</a> |
-		      <a href='?album=" . urlencode($album) . "&width=$width&screensaver=1'>screensaver</a>
-		    </td>
-		    <td width=20% align=right bgcolor=white>$next</td>
-		  </tr>
-		 </table>
-		</td></tr></table>
-	      </td></tr></table>
+<table border=0 align=center width=100%>
+  <tr>
+    <td bgcolor=#DDDDDD>
+      <table border=0 align=center width=100%>
+        <tr>
+          <td align=left width=50>$prev</td>
+          <td><p align=center><b>$links</b></p></td>
+          <td align=right width=50>$next</td>
+        </tr>
+      </table>
+    </td>
+  </tr>
+</table>
+<br/>
 	");
 }
 /****************************************************************************/
 
+
 /****************************************************************************/
 /* Print picture img */
 function print_picture($path_to_picture, $temp, $height, $alt) {
@@ -863,18 +851,21 @@ if (window.innerHeight > window.innerWidth) {
 function print_picture_in_table($path_to_picture, $temp, $height, $alt) {
 	global $BACK, $NEXT;
 	print("
-		<table border=0 width=100%><tr><td><table border=0 width=100% cellspacing=0 cellpadding=6 align=center>
-		  <tr>
-		    <td width=33% onClick=javascript:gotourl('$BACK')>&nbsp;</td>
-		    <td bgcolor=black><center>
-	");
+		<table border=0 width=100%>
+                  <tr>
+                    <td>
+                      <table border=0 width=100% cellspacing=0 cellpadding=6 align=center>
+		        <tr>
+		          <td width=33% onClick=javascript:gotourl('$BACK')>&nbsp;</td>
+		          <td bgcolor=black><center>");
 	print_picture($path_to_picture, $temp, $height, $alt);
 	print("
-		    </center></td>
+		    </center>
+                    </td>
 		    <td width=33% onClick=javascript:gotourl('$NEXT')>&nbsp;</td>
 		  </tr>
 		</table></td></tr></table>
-	");
+");
 }
 /****************************************************************************/
 
@@ -883,14 +874,14 @@ function print_picture_in_table($path_to_picture, $temp, $height, $alt) {
 /* Print lower toolbar */
 function print_lower_toolbar($resizeform, $picform, $rotateform, $width) {
 	print("
-		<table><tr><td bgcolor=white><table border=0 align=center width=$width>
+		<table align=center><tr><td bgcolor=white><table border=0 align=center width=$width>
 		  <tr align=center>
 		    <td width=33%>$resizeform</td>
 		    <td width=34%>$picform</td>
 		    <td width=33%>$rotateform</td>
 		  </tr>
 		</table>
-	");
+");
 }
 
 /****************************************************************************/
@@ -917,8 +908,7 @@ function do_flipbook_page($album, $picture, $width, $rotate, $slideshow) {
 		$picture = $pictures[0];
 	}
 
-	$descriptions = get_description("$BASEDIR/$album");
-	$picform = build_picture_form($album, $picture, $width, $slideshow, $pictures, $descriptions);
+	$picform = build_picture_form($album, $picture, $width, $slideshow, $pictures);
 	$path_to_picture = $BASEDIR . "/" . $album . "/" . $picture;
 	$path_to_picture = preg_replace("/\/+/", "/", "$path_to_picture");
 	$rotateform = build_rotate_form($album, $picture, $width, $rotate);
@@ -930,14 +920,10 @@ function do_flipbook_page($album, $picture, $width, $rotate, $slideshow) {
 	$next = get_next_link($album, $width, $pictures, $currentindex, $slideshow);
 	$back = get_back_link($album, $width, $pictures, $currentindex);
 	$alt = "";
-	if (! isset($descriptions[$picture])) {
-		$descriptions[$picture] = "";
-	}
-
 	print_picture_in_table($path_to_picture, $tempfilename, $height, $alt);
-	print_upper_toolbar($album, $descriptions[$picture], $back, $next, $width);
+	print_banner($album, "", "", $picture);
 	print_lower_toolbar($resizeform, $picform, $rotateform, $width);
-	print_picture_details($path_to_picture, $currentindex, $total, $width, $descriptions);
+	print_picture_details($path_to_picture, $currentindex, $total, $width);
 }
 /****************************************************************************/
 
@@ -958,14 +944,37 @@ function do_slideshow_page($album, $picture, $width, $slideshow) {
 	$path_to_picture = preg_replace("/\/+/", "/", "$path_to_picture");
 	$tempfilename = do_resize_picture($path_to_picture, $width, $height, $rotate);
 	print_header(0);
-	print("<body bgcolor=black topmargin=0 leftmargin=0><center><table height=100% cellpadding=0 cellspacing=0><tr><td>");
+	print("
+<body bgcolor=black topmargin=0 leftmargin=0><center>
+<table height=100% cellpadding=0 cellspacing=0>
+  <tr>
+    <td>
+");
 	print_picture($path_to_picture, $tempfilename, $height, $alt);
 	$currentindex = locate_index($picture, $pictures);
 	$next = $pictures[$currentindex+1];
 	$url = "?album=" . urlencode($album) . "&picture=" . urlencode($next) . "&width=$width";
 	$resizeform = build_resize_form($path_to_picture, $album, $picture, $width, "slideshow=$slideshow");
 	$delayform = build_delay_form($path_to_picture, $album, $picture, $width, $slideshow);
-	print("</td></tr><tr><td align=center><table><tr><td align=center>$delayform</td><td align=center>$resizeform</td></tr></table></td></tr></table></center><meta http-equiv='refresh' content='$slideshow;url=$url&slideshow=$slideshow'></body></html>\n");
+	print("
+    </td>
+  </tr>
+  <tr>
+    <td align=center>
+      <table>
+        <tr>
+          <td align=center>$delayform</td>
+          <td align=center>$resizeform</td>
+        </tr>
+      </table>
+    </td>
+  </tr>
+</table>
+</center>
+<meta http-equiv='refresh' content='$slideshow;url=$url&slideshow=$slideshow'>
+</body>
+</html>
+");
 }
 /****************************************************************************/
 
@@ -1067,14 +1076,12 @@ print_header();
 if ($screensaver) {
 	screensaver($album);
 	exit;
-} elseif ($write) {
-	do_write_descriptions($album, $file, $desc);
 } elseif (!$album) {
-	do_list_albums($base);
-} elseif ($thumbs) {
-	print_thumbnails($album);
-} else {
+	do_list_albums($base, $thumbs, $page);
+} elseif ($picture) {
 	do_flipbook_page($album, $picture, $width, $rotate, $slideshow);
+} elseif ($thumbs) {
+	print_thumbnails($album, $thumbs, $page);
 }
 print_footer();
 /****************************************************************************/
